<?php

/*
|--------------------------------------------------------------------------
| Test Case
|--------------------------------------------------------------------------
|
| The closure you provide to your test functions is always bound to a specific PHPUnit test
| case class. By default, that class is "PHPUnit\Framework\TestCase". Of course, you may
| need to change it using the "pest()" function to bind a different classes or traits.
|
*/

use App\Models\User;
use Illuminate\Database\Eloquent\Model;
use Pest\Expectation;
use Tests\TestCase;
use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertModelExists;

pest()
    ->extend(Tests\TestCase::class)
    ->use(Illuminate\Foundation\Testing\RefreshDatabase::class)
    ->in('Feature');

arch()->preset()->laravel();

/*
|--------------------------------------------------------------------------
| Expectations
|--------------------------------------------------------------------------
|
| When you're writing tests, you often need to check that values meet certain conditions. The
| "expect()" function gives you access to a set of "expectations" methods that you can use
| to assert different things. Of course, you may extend the Expectation API at any time.
|
*/

// https://github.com/defstudio/pest-plugin-laravel-expectations

expect()->extend(
    'toMatchDatabaseRecord',
    function (?string $table = null, ?string $connection = null): Expectation {
        $this->toBeInstanceOf(Model::class);

        $table = $table ?? $this->value->getTable();
        $value = $this->value->attributesToArray();

        assertDatabaseHas($table, $value, $connection);

        return $this;
    }
);

expect()->extend(
    'toBeInDatabaseExactly',
    function (?string $table = null, ?string $connection = null): Expectation {
        assertModelExists($this->value);

        return $this->toMatchDatabaseRecord();
    }
);

expect()->pipe(
    'toMatchObject',
    function (Closure $next, mixed $expected) {
        if ($expected instanceof Model) {
            return expect($this->value)
                ->toMatchObject($expected->attributesToArray());
        }

        return $next;
    }
);

/*
|--------------------------------------------------------------------------
| Functions
|--------------------------------------------------------------------------
|
| While Pest is very powerful out-of-the-box, you may have some testing code specific to your
| project that you don't want to repeat in every file. Here you can also expose helpers as
| global functions to help you to reduce the number of lines of code in your test files.
|
*/

function asAdmin(): TestCase
{
    return test()->actingAs(User::factory()->create());
}
